import pytest
from .helpers import copy_fixtures, tdenv, touch
from tradedangerous.tradedb import TradeDB


# def pytest_sessionstart(session):
#     # setup_stuff
#     copy_fixtures(tdenv.dataDir)

@pytest.fixture(scope="module")
def tdb():
    return TradeDB()

@pytest.fixture(scope="module")
def touchdb():
    return touch(tdenv.dataDir, 'TradeDangerous.db')

def pytest_addoption(parser):
    parser.addoption(
        "--runslow", action="store_true", default=False, help="run slow tests"
    )


def pytest_collection_modifyitems(config, items):
    if config.getoption("--runslow"):
        # --runslow given in cli: do not skip slow tests
        return
    skip_slow = pytest.mark.skip(reason="need --runslow option to run")
    skip_superslow = pytest.mark.skip(reason="need --runsuperslow option to run")
    for item in items:
        if "slow" in item.keywords:
            item.add_marker(skip_slow)
        if "superslow" in item.keywords:
            item.add_marker(skip_superslow)



import pytest

from tradedangerous import utils
from tradedangerous import TradeEnv


class TestUtils(object):
    # TODO: Test 'von' etc.
    
    def test_titleFixup_s(self):
        assert "Smith's" == utils.titleFixup("smith's")
    
    def test_titleFixup_mc(self):
        assert 'McDonald' == utils.titleFixup('mcdonald')
        assert 'McKilroy' == utils.titleFixup('mckilroy')
    
    def test_titleFixup_mac(self):
        assert 'MacNair' == utils.titleFixup('macnair')
        # Needs to be > 3 characters after Mac
        assert 'Macnai' == utils.titleFixup('macnai')


    def test_checkForOcrDerp(self, capsys):
        tdenv = TradeEnv()
        utils.checkForOcrDerp(tdenv, 'some', 'Aquire0')
        captured = capsys.readouterr()
        assert "Ignoring 'some/Aquire0' because it looks like OCR derp." in captured.out



import os

import pytest

from tradedangerous.cli import trade
from tradedangerous.plugins import eddblink_plug as module
from .helpers import copy_fixtures, tdfactory

PROG = "trade"
tdb = None
tdenv = None


def setup_module():
    global tdb
    global tdenv
    copy_fixtures()
    tdb, tdenv = tdfactory()


def teardown_module():
    tdb.close()


class TestTradeImportEddblink(object):
    def test_create_instance(self, monkeypatch):
        plug = module.ImportPlugin(tdb, tdenv)
        assert module.UPGRADES == "modules.json"
        assert os.path.join('data', 'eddb') in str(plug.dataPath)
        
        monkeypatch.setitem(os.environ, 'TD_EDDB', '/my/testdir')
        plug = module.ImportPlugin(tdb, tdenv)
        assert os.path.join('my', 'testdir') in str(plug.dataPath)
    
    @pytest.mark.slow
    def test_upgrades(self, capsys):
        plug = module.ImportPlugin(tdb, tdenv)
        assert module.UPGRADES == "modules.json"
        assert os.path.join('data', 'eddb') in str(plug.dataPath)
        plug.downloadFile(module.UPGRADES, plug.upgradesPath)
        assert (plug.dataPath / plug.upgradesPath).is_file()


    @pytest.mark.superslow
    def test_import_clean(self, capsys):
        trade([PROG, "import", "-P=eddblink", '--opt=clean,skipvend,force'])
        captured = capsys.readouterr()
        # with capsys.disabled():
        #     print("Here")
        #     print(captured.out)
        #     print("to Here")
        assert "NOTE: Import completed." in captured.out
    
    @pytest.mark.superslow
    def test_import_upgrade(self, capsys):
        trade([PROG, "import", "-P=eddblink", '--opt=upgrade'])
        captured = capsys.readouterr()
        with capsys.disabled():
            print("Here")
            print(captured.out)
            print("to Here")
        assert "NOTE: Import completed." in captured.out



import sys
import os
import re
from time import sleep
from io import StringIO
from pathlib import Path
from contextlib import contextmanager

from tradedangerous.tradedb import TradeDB
from tradedangerous import fs, TradeEnv

__all__ = ['tdenv', 'captured_output', 'is_initialized']

_ROOT = os.path.abspath(os.path.dirname(__file__))
tdenv = TradeEnv(debug=3)

@contextmanager
def captured_output():
    new_out, new_err = StringIO(), StringIO()
    old_out, old_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = new_out, new_err
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout, sys.stderr = old_out, old_err

@contextmanager
def replace_stdin(target):
    orig = sys.stdin
    sys.stdin = target
    yield
    sys.stdin = orig


def tdfactory():
    """Generates a usable environment with debug
    Returns
    -------
    [TradeEnv, TradeDB]
        list with instances of TradeEnv and TradeDB
    """
    return [TradeDB(tdenv=tdenv, load=True, debug=3), tdenv]


def file_exists(filename):
    return Path(filename).is_file()

def is_initialized():
    return Path(tdenv.dataDir, 'TradeDangerous.db').is_file()

def empty_path(p):
    """Deletes a directory tree including files
    """
    if p.exists() and p.is_dir():
        # print("cleaning up", p)
        for entry in p.glob('*'):
            if entry.is_file():
                # print("removing file", entry)
                entry.unlink()
            elif entry.is_dir() and entry != p:
                empty_path(entry)
        # print("removing directory", entry)
        p.rmdir()
    elif p.is_file():
        # print("removing file p", entry)
        p.unlink()


def remove_fixtures(toDir=None):
    if not toDir:
        toDir = tdenv.dataDir
    toPath = Path(toDir)
    empty_path(toPath)


def copy_fixtures(toDir=None):
    if not toDir:
        toDir = tdenv.dataDir
    toPath = Path(toDir)
    print("will copy fixures to {}".format(toPath))
    print("but first clean old")
    empty_path(toPath)
    fs.ensurefolder(toDir)
    
    fs.copyallfiles(tdenv.templateDir, tdenv.dataDir)
    fs.copyallfiles(Path(_ROOT, 'fixtures'), tdenv.dataDir)
    fs.ensureflag(Path(tdenv.dataDir, '.tddata'))
    touch(Path(tdenv.dataDir, 'TradeDangerous.db'))


def touch(*args):
    filename = Path(*args)
    return fs.touch(filename)

class pytest_regex:
    """Assert that a given string meets some expectations."""
    
    def __init__(self, pattern, flags=0):
        self._regex = re.compile(pattern, flags)
    
    def __eq__(self, actual):
        return bool(self._regex.match(actual))
    
    def __repr__(self):
        return self._regex.pattern

def regex_findin(pattern, value):
    return bool(re.search(pattern, value))
